@@ -1,10 +1,11 @@
 #encoding: utf8
 
 import os, gettext, time, copy, sys, re
-
+import traceback
 import unit_tree.helpers as helpers
 import wesnoth.wmlparser2 as wmlparser2
 
+
 pics_location = "../../pics"
 
 html_header = '''
@@ -34,7 +35,7 @@ html_footer = '''
 <div id="footer">
 <p>%(generation_note)s</p>
 <p><a href="http://wiki.wesnoth.org/Site_Map">Site map</a></p>
-<p><a href="http://www.wesnoth.org/wiki/Wesnoth:Copyrights">Copyright</a> &copy; 2003-2012 The Battle for Wesnoth</p>
+<p><a href="http://www.wesnoth.org/wiki/Wesnoth:Copyrights">Copyright</a> &copy; 2003&#8211;2015 The Battle for Wesnoth</p>
 <p>Supported by <a href="http://www.jexiste.fr/">Jexiste</a></p>
 </div>
 </div>
@@ -186,6 +187,7 @@ class HTMLOutput:
         forest = self.forest = helpers.UnitForest()
         units_added = {}
         for uid, u in self.wesnoth.unit_lookup.items():
+            if u.hidden: continue
             if grouper.unitfilter(u):
                 forest.add_node(helpers.UnitNode(u))
                 units_added[uid] = u
@@ -242,9 +244,9 @@ class HTMLOutput:
         rows_count = breadth + len(thelist)
         # Create empty grid.
         rows = []
-        for j in range(rows_count):
+        for j in xrange(rows_count):
             column = []
-            for i in range(6):
+            for i in xrange(6):
                 column.append((1, 1, None))
             rows.append(column)
 
@@ -264,7 +266,7 @@ class HTMLOutput:
                 if level < 0: level = 0
                 if level > 5: level = 5
                 rows[x][level] = (1, node.breadth, node)
-                for i in range(1, node.breadth):
+                for i in xrange(1, node.breadth):
                     rows[x + i][level] = (0, 0, node)
                 grid_place(node.children, x)
                 x += node.breadth
@@ -276,7 +278,7 @@ class HTMLOutput:
             node.name = grouper.group_name(group)
 
             rows[x][0] = (6, 1, node)
-            for i in range(1, 6):
+            for i in xrange(1, 6):
                 rows[x][i] = (0, 0, None)
             nodes = groups[group]
             x += 1
@@ -308,7 +310,7 @@ class HTMLOutput:
         def abbrev(name):
             abbrev = name[0]
             word_seperators = [" ", "_", "+", "(", ")"]
-            for i in range(1, len(name)):
+            for i in xrange(1, len(name)):
                 if name[i] in ["+", "(", ")"] or name[i - 1] in word_seperators and name[i] not in word_seperators:
                     abbrev += name[i]
             return abbrev
@@ -374,7 +376,7 @@ class HTMLOutput:
             add_menu("races_menu", x)
 
             for row in self.unitgrid:
-                for column in range(6):
+                for column in xrange(6):
                     hspan, vspan, un = row[column]
                     if not un: continue
                     if isinstance(un, helpers.GroupNode):
@@ -416,6 +418,7 @@ class HTMLOutput:
                     self.target, r, r))
                 for uid in races[r]:
                     un = self.wesnoth.unit_lookup[uid]
+                    if un.hidden: continue
                     if "mainline" in un.campaigns: addon = "mainline"
                     else: addon = self.addon
                     link = "../../%s/%s/%s.html" % (addon, self.isocode, uid)
@@ -439,8 +442,8 @@ class HTMLOutput:
             col += 1
             write("<td>")
             labb = lang
-            underscore = labb.find("_")
-            if underscore > 0: labb = labb[:underscore]
+            #underscore = labb.find("_")
+            #if underscore > 0: labb = labb[:underscore]
             if self.addon == "mainline":
                 write(" <a title=\"%s\" href=\"../%s/%s\">%s</a><br/>\n" % (
                     languages[lang], lang, self.target,
@@ -491,7 +494,11 @@ class HTMLOutput:
                 "Warning: Missing image for unit %s(%s).\n" % (
                 u.get_text_val("id"), x.name))
             return None, None
-        picname = image_collector.add_image(self.addon, image)
+        icpic = image_collector.add_image_check(self.addon, image)
+        if not icpic.ipath:
+            error_message("Warning: No picture %s for unit %s.\n" %
+                (image, u.get_text_val("id")))
+        picname = icpic.id_name
         image = os.path.join(pics_location, picname)
         if portrait:
             picname = image_collector.add_image(self.addon, portrait,
@@ -544,7 +551,7 @@ class HTMLOutput:
         rows = self.unitgrid
         write("<table class=\"units\">\n")
         write("<colgroup>")
-        for i in range(6):
+        for i in xrange(6):
             write("<col class=\"col%d\" />" % i)
         write("</colgroup>")
 
@@ -552,9 +559,9 @@ class HTMLOutput:
             "../../../images/misc/leader-crown.png", no_tc=True)
         crownimage = os.path.join(pics_location, pic)
         ms = None
-        for row in range(len(rows)):
+        for row in xrange(len(rows)):
             write("<tr>\n")
-            for column in range(6):
+            for column in xrange(6):
                 hspan, vspan, un = rows[row][column]
                 if vspan:
                     attributes = ""
@@ -751,6 +758,7 @@ class HTMLOutput:
             if not fportrait: fportrait = portrait
             write('<img src="%s" alt="(image)" />\n' % mimage)
             write('<img src="%s" alt="(image)" />\n' % fimage)
+            image = mimage
         else:
             image, portrait = self.pic(unit, unit)
             write('<img src="%s" alt="(image)" />\n' % image)
@@ -767,6 +775,8 @@ class HTMLOutput:
         hp = uval("hitpoints")
         mp = uval("movement")
         xp = uval("experience")
+        vision = uval("vision")
+        jamming = uval("jamming")
         level = uval("level")
         alignment = uval("alignment")
 
@@ -813,14 +823,17 @@ class HTMLOutput:
             ("cost", _("Cost: ", "wesnoth-help")),
             ("hitpoints", _("HP: ")),
             ("movement", _("Movement", "wesnoth-help") + ": "),
+            ("vision", _("Vision", "wesnoth-help") + ": "),
+            ("jamming", _("Jamming", "wesnoth-help") + ":"),
             ("experience", _("XP: ")),
             ("level", _("Level") + ": "),
             ("alignment", _("Alignment: ")),
             ("id", "ID")]:
+            x = uval(val)
+            if not x and val in ("jamming", "vision"): continue
+            if val == "alignment": x = _(x)
             write("<tr>\n")
             write("<td>%s</td>" % text)
-            x = uval(val)
-            if val == "alignment": x = _(x)
             write("<td class=\"val\">%s</td>" % x)
             write("</tr>\n")
 
@@ -849,14 +862,14 @@ class HTMLOutput:
             if not icon:
                 icon = "attacks/%s.png" % aid
 
-            image = image_collector.add_image_check(self.addon,
+            image_add = image_collector.add_image_check(self.addon,
                 icon, no_tc = True)
-            if not image.ipath:
+            if not image_add.ipath:
                 error_message("Error: No attack icon '%s' found for '%s'.\n" % (
                     icon, uid))
                 icon = os.path.join(pics_location, "unit$elves-wood$shaman.png")
             else:
-                icon = os.path.join(pics_location, image.id_name)
+                icon = os.path.join(pics_location, image_add.id_name)
             write("<td><img src=\"%s\" alt=\"(image)\"/></td>" % icon)
 
             write("<td><b>%s</b>" % aname)
@@ -931,14 +944,22 @@ class HTMLOutput:
         write('</div>')
         write('<div class="unit-column-right">')
 
-        if portrait:
-            write('<div class="portrait">')
-            if female:
-                write('<img width="200" src="%s" alt="(portrait)" />\n' % portrait)
-                write('<img width="200" src="%s" alt="(portrait)" />\n' % fportrait)
+        for si in xrange(2):
+            if si and not female: break
+            if si:
+                sportrait = fportrait
+                simage = fimage
             else:
-                write('<img width="200" src="%s" alt="(portrait)" />\n' % portrait)
-            write('</div>\n')
+                simage = image
+                sportrait = portrait
+
+            style = "background-image: url(%s);" % simage
+
+            write('<div class="portrait">')
+            write('<div style="%s">&nbsp;</div>' % style)
+            if portrait:
+                write('<img src="%s" alt="(portrait)" />\n' % sportrait)
+            write('</div>')
 
         # Write info about movement costs and terrain defense.
         write("<h2>" + _("Terrain", "wesnoth-help") + "</h2>\n")
@@ -962,12 +983,32 @@ class HTMLOutput:
             already[tid] = 1
             name = T(t, "name")
             ticon = t.get_text_val("symbol_image")
-            if tid == "flat": ticon = "grass/green-symbol"
-            elif tid == "frozen": ticon = "frozen/ice"
-            elif tid == "unwalkable": ticon = "unwalkable/lava"
-            elif tid == "village": ticon = "village/human-tile"
+            if not ticon:
+                ticon = t.get_text_val("icon_image")
+
+            # Use nice images for known mainline terrain types
+            if tid == "fungus": ticon = "forest/mushrooms-tile"
+            elif tid == "cave": ticon = "cave/floor6"
+            elif tid == "sand": ticon = "sand/beach"
+            elif tid == "reef": ticon = "water/reef-tropical-tile"
+            elif tid == "hills": ticon = "hills/regular"
+            elif tid == "swamp_water": ticon = "swamp/water-tile"
+            elif tid == "shallow_water": ticon = "water/coast-tile"
+            elif tid == "castle": ticon = "castle/castle-tile"
+            elif tid == "mountains": ticon = "mountains/snow-tile"
+            elif tid == "deep_water": ticon = "water/ocean-tile"
+            elif tid == "flat": ticon = "grass/green-symbol"
             elif tid == "forest": ticon = "forest/pine-tile"
-            terrainlist.append((name, tid, ticon))
+            elif tid == "frozen": ticon = "frozen/ice"
+            elif tid == "village": ticon = "village/human-tile"
+            elif tid == "impassable": ticon = "void/void"
+            elif tid == "unwalkable": ticon = "unwalkable/lava"
+            elif tid == "rails": ticon = "misc/rails-ne-sw"
+            
+            if ticon:
+                terrainlist.append((name, tid, ticon))
+            else:
+                error_message("Terrain " + tid + " has no symbol_image\n")
         terrainlist.sort()
 
         for tname, tid, ticon in terrainlist:
@@ -979,7 +1020,13 @@ class HTMLOutput:
             not_from_race, d = find_attr("defense", tid)
 
             if d == "-": d = 100
-            try: d = "%d%%" % (100 - int(d))
+
+            try:
+                d = int(d)
+                # negative defense has something to do with best defense if
+                # there's multiple terrain types
+                if d < 0: d = -d
+                d = "%d%%" % (100 - d)
             except ValueError:
                 error_message("Warning: Invalid defense %s for %s.\n" % (
                     d, uid))
@@ -1065,15 +1112,24 @@ def generate_single_unit_reports(addon, isocode, wesnoth):
     html.analyze_units(grouper, True)
 
     for uid, unit in wesnoth.unit_lookup.items():
+        if unit.hidden: continue
         if "mainline" in unit.campaigns and addon != "mainline": continue
-        filename = os.path.join(path, "%s.html" % uid)
-
-        # We probably can come up with something better.
-        if os.path.exists(filename):
-            age = time.time() - os.path.getmtime(filename)
-            # was modified in the last 12 hours - we should be ok
-            if age < 3600 * 12: continue
+        
+        try:
+            htmlname = u"%s.html" % uid
+            filename = os.path.join(path, htmlname).encode("utf8")
 
+            # We probably can come up with something better.
+            if os.path.exists(filename):
+                age = time.time() - os.path.getmtime(filename)
+                # was modified in the last 12 hours - we should be ok
+                if age < 3600 * 12: continue
+        except (UnicodeDecodeError, UnicodeEncodeError) as e:
+            traceback.print_exc()
+            error_message("Unicode problem: " + repr(path) + " + " + repr(uid) + "\n")
+            error_message(str(e) + "\n")
+            continue
+        
         output = MyFile(filename, "w")
         html.target = "%s.html" % uid
         html.write_unit_report(output, unit)
@@ -1081,7 +1137,7 @@ def generate_single_unit_reports(addon, isocode, wesnoth):
         
 def html_postprocess_file(filename, isocode, batchlist):
 
-    print("postprocessing " + filename)
+    print(u"postprocessing " + repr(filename))
     
     chtml = u""
     ehtml = u""
@@ -1100,7 +1156,7 @@ def html_postprocess_file(filename, isocode, batchlist):
             else:
                 cids[1].append(c)
 
-    for i in range(2):
+    for i in xrange(2):
         
         campaigns = cids[i]
         campaigns.sort(key = lambda x: "A" if x[1] == "mainline" else "B" + x[2])
@@ -1127,8 +1183,7 @@ def html_postprocess_file(filename, isocode, batchlist):
             else:
                 eids[1].append(e)
             
-            
-    for i in range(2):
+    for i in xrange(2):
         eras = eids[i]
         eras.sort(key = lambda x: x[2])
 
